--[[
	Free Bag Slots

	By sarf

	This mod allows the lazy user to keep track of the number of free slots in his or her backpacks.

	Thanks goes to Beider for suggesting this.
	
	CosmosUI URL:
	http://www.cosmosui.org/forums/viewtopic.php?t=
	
   ]]


-- Constants

-- Variables
FreeBagSlots_Enabled = 0;

FreeBagSlots_Number_Color_Red = 1.0;
FreeBagSlots_Number_Color_Green = 1.0;
FreeBagSlots_Number_Color_Blue = 1.0;

FreeBagSlots_Saved_Hooked_Function = nil;
FreeBagSlots_Cosmos_Registered = 0;

-- executed on load, calls general set-up functions
function FreeBagSlots_OnLoad()
	FreeBagSlots_Register();
end

-- registers the mod with Cosmos
function FreeBagSlots_Register_Cosmos()
	if ( ( Cosmos_RegisterConfiguration ) and ( FreeBagSlots_Cosmos_Registered == 0 ) ) then
		Cosmos_RegisterConfiguration(
			"COS_FREEBAGSLOTS",
			"SECTION",
			TEXT(FREEBAGSLOTS_CONFIG_HEADER),
			TEXT(FREEBAGSLOTS_CONFIG_HEADER_INFO)
		);
		Cosmos_RegisterConfiguration(
			"COS_FREEBAGSLOTS_HEADER",
			"SEPARATOR",
			TEXT(FREEBAGSLOTS_CONFIG_HEADER),
			TEXT(FREEBAGSLOTS_CONFIG_HEADER_INFO)
		);
		Cosmos_RegisterConfiguration(
			"COS_FREEBAGSLOTS_ENABLED",
			"CHECKBOX",
			TEXT(FREEBAGSLOTS_ENABLED),
			TEXT(FREEBAGSLOTS_ENABLED_INFO),
			FreeBagSlots_Toggle_Enabled,
			FreeBagSlots_Enabled
		);
		FreeBagSlots_Cosmos_Registered = 1;
	end
end

-- registers the mod with the system, integrating it with slash commands and "master" AddOns
function FreeBagSlots_Register()
	if ( Cosmos_RegisterConfiguration ) then
		FreeBagSlots_Register_Cosmos();
	else
		SlashCmdList["FREEBAGSLOTSSLASHENABLE"] = FreeBagSlots_Enable_ChatCommandHandler;
		SLASH_FREEBAGSLOTSSLASHENABLE1 = "/freebagslotstoggle";
		SLASH_FREEBAGSLOTSSLASHENABLE2 = "/fbstoggle";
		SLASH_FREEBAGSLOTSSLASHENABLE3 = "/freebagslotsenable";
		SLASH_FREEBAGSLOTSSLASHENABLE4 = "/fbsenable";
		SLASH_FREEBAGSLOTSSLASHENABLE5 = "/freebagslotsdisable";
		SLASH_FREEBAGSLOTSSLASHENABLE6 = "/fbsdisable";
		this:RegisterEvent("VARIABLES_LOADED");
	end

	if ( Cosmos_RegisterChatCommand ) then
		local FreeBagSlotsEnableCommands = {"/freebagslotstoggle","/fbstoggle","/freebagslotsenable","/fbsenable","/freebagslotsdisable","/fbsdisable"};
		Cosmos_RegisterChatCommand (
			"FREEBAGSLOTS_ENABLE_COMMANDS", -- Some Unique Group ID
			FreeBagSlotsEnableCommands, -- The Commands
			FreeBagSlots_Enable_ChatCommandHandler,
			FREEBAGSLOTS_CHAT_COMMAND_ENABLE_INFO -- Description String
		);
	end
	this:RegisterEvent("BAG_UPDATE");
	this:RegisterEvent("UNIT_INVENTORY_CHANGED");
end

-- Handles chat - e.g. slashcommands - enabling/disabling the FreeBagSlots
function FreeBagSlots_Enable_ChatCommandHandler(msg)
	msg = string.lower(msg);
	
	-- Toggle appropriately
	if ( (string.find(msg, 'on')) or ((string.find(msg, '1')) and (not string.find(msg, '-1')) ) ) then
		FreeBagSlots_Toggle_Enabled(1);
	else
		if ( (string.find(msg, 'off')) or (string.find(msg, '0')) ) then
			FreeBagSlots_Toggle_Enabled(0);
		else
			FreeBagSlots_Toggle_Enabled(-1);
		end
	end
end

-- Handles events
function FreeBagSlots_OnEvent(event)
	if ( event == "VARIABLES_LOADED" ) then
		if ( FreeBagSlots_Cosmos_Registered == 0 ) then
			local value = getglobal("COS_FREEBAGSLOTS_ENABLED_X");
			if (value == nil ) then
				-- defaults to off
				value = 0;
			end
			FreeBagSlots_Toggle_Enabled(value);
		end
	end
	if ( event == "BAG_UPDATE") then
		FreeBagSlots_UpdateFreeBagSlots();
	end
	if ( event == "UNIT_INVENTORY_CHANGED" ) then
		FreeBagSlots_UpdateFreeBagSlots();
	end
end

function FreeBagSlots_GetFreeBagSlots(bag)
	local freeSlots = 0;
	local slot;
	for slot = GetContainerNumSlots(bag),1,-1 do
		if ( not SlotHasItem(bag, slot ) ) then
			freeSlots = freeSlots + 1;
		end
	end
	return freeSlots;
end

function FreeBagSlots_GetTotalBagSlots(bag)
	return GetContainerNumSlots(bag);
end

function FreeBagSlots_GetByteValue(pValue)
	local value = tonumber(pValue);
	if ( value <= 0 ) then return 0; end
	if ( value >= 255 ) then return 255; end
	return value;
end

-- Yet another function from George Warner, modified a bit to fit my own nefarious purposes. 
-- It can now accept r, g and b specifications, too (leaving out a), as well as handle 255 255 255
-- Source : http://www.cosmosui.org/cgi-bin/bugzilla/show_bug.cgi?id=159
function FreeBagSlots_GetColorFormatString(a, r, g, b)
	local percent = false;
	if ( ( ( not b ) or ( b <= 1 ) ) and ( a <= 1 ) and ( r <= 1 ) and ( g <= 1) ) then percent = true; end
	if ( ( not b ) and ( a ) and ( r ) and ( g ) ) then b = g; g = r; r = a; if ( percent ) then a = 1; else a = 255; end end
	if ( percent ) then a = a * 255; r = r * 255; g = g * 255; b = b * 255; end
	a = FreeBagSlots_GetByteValue(a); r = FreeBagSlots_GetByteValue(r); g = FreeBagSlots_GetByteValue(g); b = FreeBagSlots_GetByteValue(b);
	
	return format("|c%02X%02X%02X%02X%%s|r", a, r, g, b);
end

function FreeBagSlots_GetColorFormat()
	return FreeBagSlots_GetColorFormatString(FreeBagSlots_Number_Color_Red, FreeBagSlots_Number_Color_Green, FreeBagSlots_Number_Color_Blue);
end

function FreeBagSlots_GetFreeBagSlotsText(bag)
	local totalSlots = FreeBagSlots_GetTotalBagSlots(bag);
	if ( totalSlots <= 0 ) then
		return "";
	else
		local freeSlots = FreeBagSlots_GetFreeBagSlots(bag);
		local text = "";
		text = freeSlots.."";
		--text = freeSlots.."/"..totalSlots;
		return format(FreeBagSlots_GetColorFormat(), text);
	end
end

function FreeBagSlots_UpdateFreeBagSlotsCount(name, text, disable)
	local obj = getglobal(name);
	local objFrame = getglobal(name.."Frame");
	if ( obj ) then
		obj:SetText(text);
		if ( objFrame ) then
			objFrame:SetScale(1.0);
			if ( not disable ) then
				objFrame:Show();
			else
				objFrame:Hide();
			end
		end
		if ( not disable ) then
			obj:Show();
		else
			obj:Hide();
		end
	end
end

function FreeBagSlots_UpdateFreeBagSlots(disable)
	FreeBagSlots_UpdateFreeBagSlotsCount("MainMenuBarBackpackButtonFreeSlotCount", FreeBagSlots_GetFreeBagSlotsText(0), disable);
	local name, obj;
	for i = 0, 3 do
		name = "CharacterBag"..i.."SlotFreeSlotCount";
		
		FreeBagSlots_UpdateFreeBagSlotsCount(name, FreeBagSlots_GetFreeBagSlotsText(i+1), disable);
	end
end

-- Toggles the enabled/disabled state of an option and returns the new state
--  if toggle is 1, it's enabled
--  if toggle is 0, it's disabled
--   otherwise, it's toggled
function FreeBagSlots_Generic_Toggle(toggle, variableName, CVarName, enableMessage, disableMessage, CosmosVarName)
	local oldvalue = getglobal(variableName);
	local newvalue = toggle;
	if ( ( toggle ~= 1 ) and ( toggle ~= 0 ) ) then
		if (oldvalue == 1) then
			newvalue = 0;
		elseif ( oldvalue == 0 ) then
			newvalue = 1;
		else
			newvalue = 0;
		end
	end
	setglobal(variableName, newvalue);
	setglobal(CVarName, newvalue);
	if ( newvalue ~= oldvalue ) then
		if ( newvalue == 1 ) then
			FreeBagSlots_Print(TEXT(getglobal(enableMessage)));
		else
			FreeBagSlots_Print(TEXT(getglobal(disableMessage)));
		end
	end
	FreeBagSlots_Register_Cosmos();
	if ( FreeBagSlots_Cosmos_Registered == 0 ) then 
		RegisterForSave(variableName);
	else
		if ( CosmosVarName ) then
			Cosmos_UpdateValue(strsub(CVarName, 1, strlen(CVarName)-2), CSM_CHECKONOFF, newvalue);
		end
	end
	return newvalue;
end

-- Toggles the enabled/disabled state of the FreeBagSlots
--  if toggle is 1, it's enabled
--  if toggle is 0, it's disabled
--   otherwise, it's toggled
function FreeBagSlots_Toggle_Enabled(toggle)
	local newvalue = FreeBagSlots_Generic_Toggle(toggle, "FreeBagSlots_Enabled", "COS_FREEBAGSLOTS_ENABLED_X", "FREEBAGSLOTS_CHAT_ENABLED", "FREEBAGSLOTS_CHAT_DISABLED");
	FreeBagSlots_UpdateFreeBagSlots();
end

-- Prints out text to a chat box.
function FreeBagSlots_Print(msg,r,g,b,frame,id,unknown4th)
	if(unknown4th) then
		local temp = id;
		id = unknown4th;
		unknown4th = id;
	end
				
	if (not r) then r = 1.0; end
	if (not g) then g = 1.0; end
	if (not b) then b = 1.0; end
	if ( frame ) then 
		frame:AddMessage(msg,r,g,b,id,unknown4th);
	else
		if ( DEFAULT_CHAT_FRAME ) then 
			DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b,id,unknown4th);
		end
	end
end
